/*
 * The code file for the MagicTutu GUI routines.
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU LGPL version 2.
 *
 * See the file "COPYING.LIB" for software license details.
 */

#include "def.h"

/* Screen dimensions */
int  scrn_cntr,     /* Centre of Screen */
     scrn_widt,     /* Screen Width */
     scrn_hght,     /* Screen Height */
     clr_mode = 8;  /* Colour Mode */
volatile long long gui_timer = 0; /* GUI internal timer (64-bit) */
/* Screen Capture Counts */
int screen_capture_counts = 0;
/* BIOS font height */
int biosfontheight = 0;

RGB *set_gui_palette;

#define gui_timer_rate  0x0100 /* Service GUI timer at each 1024 ticks */

/* The 8x8 bios font */
FONT *biosfont;
/* The Wallpaper */
BITMAP *wallpaper = NULL;

/* The Mouse Button Status */
int mouse_status, mouse_dclick = 0, int_mouse_x = -1, mouse_type,
    int_mouse_y = -1, org_mouse_x, org_mouse_y, mouse_first_state;
int drag_setting = 4, dclick_time = 596590, /* half a second (1193181 / 2) */
    dclick_maxstray = 4;
volatile int timer_dclick = -1;
bool click_guard = FALSE, drag_flag = FALSE;
/* GUI flags */
bool window_relocate= FALSE;

/* List of Window Handle Data */
WINSTRUCT windata [MAX_WINHANDLES]; /* Window Information */
int winhandles [MAX_WINHANDLES],    /* List of handles that are in use */
    wininpos = 0,                   /* Handles are in use */
    winlastactive = -1,             /* Last active window */
    winactive = -1;                 /* Active Window */

/* Active Drag Focus Control */
int drag_focus = -1,
    (*drag_control)(int handle, void *value) = NULL,
    (*drag_termin)(int handle, void *value) = NULL;

/* The colour palettes */
char *colornames [] = {"black", "dark blue", "dark green", "dark cyan", "red",
                       "purple", "brown", "light grey", "dark grey", "blue",
                       "green", "cyan", "pink", "magenta", "yellow", "white",
                       "orange", "olive"};
RGB rgbcolors [] = {
  {0x00, 0x00, 0x00},  /* Black */
  {0x00, 0x00, 0xaa},  /* Dark Blue */
  {0x00, 0xaa, 0x00},  /* Dark Green */
  {0x00, 0xaa, 0xaa},  /* Dark Cyan */
  {0xaa, 0x00, 0x00},  /* Red */
  {0xaa, 0x00, 0xaa},  /* Purple */
  {0xaa, 0x55, 0x00},  /* Brown */
  {0xaa, 0xaa, 0xaa},  /* Light Grey */
  {0x55, 0x55, 0x55},  /* Dark Grey */
  {0x55, 0x55, 0xff},  /* Blue */
  {0x55, 0xff, 0x55},  /* Green */
  {0x55, 0xff, 0xff},  /* Cyan */
  {0xff, 0x55, 0x55},  /* Pink */
  {0xff, 0x55, 0xff},  /* Magenta */
  {0xff, 0xff, 0x55},  /* Yellow */
  {0xff, 0xff, 0xff},  /* White */
  {0xff, 0xaa, 0x00},  /* Orange */
  {0xaa, 0xaa, 0x00}}; /* Olive */

int pal_stndrd [18];

/* The GUI Colour Scheme Palette */

char *clrschemenames [] = {"Wallpaper",
                           "Window Background",
                           "Window Foreground",
                           "GUI Button Outline",
                           "GUI Button Body",
                           "GUI Button Text",
                           "Window Border"
                           "Window Title",
                           "Window Border II",
                           "GUI Button Outline II"
                           "Window Bar Icons (Foreground)",
                           "Window Bar Icons (Background)"
                           "Window Border (Inactive)",
                           "Window Title (Inactive)",
                           "Window Border II (Inactive)"
                           "GUI Botton Outline (focus)",
                           "GUI Button Body (focus)",
                           "GUI Button Text (focus)",
                           "GUI Button Outline II (focus)"
                           "GUI Check Box Outline",
                           "GUI Check Box Body",
                           "GUI Check Box Mark",
                           "GUI Check Box Outline (focus)",
                           "GUI Check Box Body (focus)",
                           "GUI Check Box Mark (focus)",
                           "GUI Radio Button Outline",
                           "GUI Radio Button Body",
                           "GUI Radio Button Mark",
                           "GUI Radio Button Outline (focus)",
                           "GUI Radio Button Body (focus)",
                           "GUI Radio Button Mark (focus)",
                           "GUI Text Input Border",
                           "GUI Text Input Body",
                           "GUI Text Input Entry",
                           "GUI Text Input Border (focus)",
                           "GUI Text Input Body (focus)",
                           "GUI Text Input Entry (focus)",
                           "Window Subtitle Header"
                           };
RGB rgbscheme[] = {
  {0x00,0x00,0x80}, /* Wallpaper Colour */
  {0xff,0xff,0xff}, /* Window Background */
  {0x00,0x00,0x00}, /* Window Foreground */
  {0xc0,0x00,0xc0}, /* GUI Outlining Colour */
  {0x80,0x00,0x80}, /* GUI Button Colour */
  {0xff,0x80,0xff}, /* GUI Button Text */
  {0xff,0xff,0x00}, /* Window Border */
  {0x80,0x00,0x00}, /* Window Title Colour */
  {0x80,0x80,0x00}, /* Window Border II */
  {0x40,0x00,0x40}, /* GUI Outlining Colour II */
  {0x40,0xff,0x40}, /* Window Bar Icons (Foreground) */
  {0x00,0x80,0x00}, /* Window Bar Icons (Background) */
  {0x40,0x40,0x00}, /* Window Border (Disabled) */
  {0xff,0x80,0x00}, /* Window Title (Disable) */
  {0x80,0x80,0x00}, /* Window Border II (Disabled) */
  {0xe0,0x80,0xe0}, /* GUI Outlining Colour (focus) */
  {0xc0,0x80,0xc0}, /* GUI Button Colour (focus) */
  {0xff,0xc0,0xff}, /* GUI Button Text (focus) */
  {0xa0,0x80,0xa0}, /* GUI Outlining Colour II (focus) */
  {0x00,0x00,0xff}, /* GUI Check Box Outline */
  {0x00,0x00,0x80}, /* GUI Check Box Body */
  {0x80,0x80,0xff}, /* GUI Check Box Mark */
  {0x80,0x80,0xff}, /* GUI Check Box Outline (focus) */
  {0x80,0x80,0xc0}, /* GUI Check Box Body (focus) */
  {0xc0,0xc0,0xff}, /* GUI Check Box Mark (focus) */
  {0x00,0x00,0xff}, /* GUI Radio Outline */
  {0x00,0x00,0x80}, /* GUI Radio Body */
  {0x80,0x80,0xff}, /* GUI Radio Mark */
  {0x80,0x80,0xff}, /* GUI Radio Outline (focus) */
  {0x80,0x80,0xc0}, /* GUI Radio Body (focus) */
  {0xc0,0xc0,0xff}, /* GUI Radio Mark (focus) */
  {0x00,0xff,0xff}, /* GUI Text Input Border */
  {0x00,0x80,0x80}, /* GUI Text Input Body */
  {0xff,0xff,0x00}, /* GUI Text Input Entry */
  {0x00,0xff,0xff}, /* GUI Text Input Border (focus) */
  {0xff,0xff,0x00}, /* GUI Text Input Body (focus) */
  {0x00,0x80,0x80}, /* GUI Text Input Entry (focus) */
  {0x80,0x00,0x80}  /* Window Subtitle Header */
 };

int colorscheme[clrsh_maxno];

/* The RGB MAP for the colours */
RGB_MAP clr_conv_map;

/* This sets the 256 colour palette */
void make_rgb_map (RGB *pal) {
  int i;

  set_gui_palette = pal;
  /* -- if colour mode is eight -- */
   if (clr_mode == 8)
    set_palette (pal); /* Set the palette */
  /* Get closet matching colours */
   if (pal) {
    create_rgb_table (&clr_conv_map, pal, NULL);
    rgb_map = &clr_conv_map;
   }

 /* Setup the "standard" colour table hookups */
   for (i = 0; i < clr_maxno; i++) {
    pal_stndrd [i] = makecol (rgbcolors[i].r, rgbcolors[i].g, rgbcolors[i].b);
   }
 /* Create the optimized GUI palette hookups */
   for (i = 0; i < clrsh_maxno; i++) {
    colorscheme [i] = makecol (rgbscheme[i].r, rgbscheme[i].g, rgbscheme[i].b);
   }
 }

/* Service GUI timer */
 void service_gui_timer () {
  gui_timer += gui_timer_rate;
  if (timer_dclick >= 0) timer_dclick -= gui_timer_rate; /* Mouse Double Click */
 }
END_OF_FUNCTION (service_gui_timer);

/* Register single mouse click without going into the double click phase */
 void register_single_click () {
  timer_dclick = -1; mouse_dclick = 0;
 }

/* Initialize the GUI system */
 int initialize_gui () {
  int i;

  /* Start Allegro libraries */
   allegro_init ();
  /* Start GUI timer (timer routines are interrupts, thus must be locked) */
   LOCK_FUNCTION (service_gui_timer);
   LOCK_VARIABLE (gui_timer);
   install_timer ();

   gui_timer = 0; /* Reset the timer */
    if (install_int_ex(service_gui_timer, gui_timer_rate)) {
     printf("Error! Unable to initalize the timer!\n");
     exit(255);
    }
   wininpos = 0;  /* No Handles are in use */
   timer_dclick = -1; /* No Double Click Triggers */
  /* Number each window handle */
    for (i = 0; i < MAX_WINHANDLES; i++) {
     winhandles[i] = i;
    }
  /* Number each TBMO handle */
   tbmo_num = 0;
    for (i = 0; i < MAX_GUI_TBMOS; i++) {
     tbmo_handle[i] = i;
    }

   biosfont = font;
   biosfontheight = text_height (biosfont);

  /* Install Keyboard */
   install_keyboard ();
  /* Install Allegro Keyboard Driver */
    if ((mouse_type = install_mouse ()) < 1) {
     printf("Error! Mouse Driver is not installed!\n");
     exit(255);
    }
  /* Keep mouse hidden */
   show_mouse (NULL);
  return 0; /* returns non-zero number if GUI can not be initialized */
 }

/* Check if two rectangles do overlap */
static inline
       check_for_overlap (int x_0, int y_0, int h_0, int v_0,
                          int x_1, int y_1, int h_1, int v_1) {
 int xx0 = x_0 + h_0, xx1 = x_1 + h_1,
     yy0 = y_0 + v_0, yy1 = y_1 + v_1;

 return !(
          (x_0 <= x_1 && x_0 <= xx1 && xx0 <= x_1 && xx0 <= xx1) ||
          (x_0 >= x_1 && x_0 >= xx1 && xx0 >= x_1 && xx0 >= xx1) ||
          (y_0 <= y_1 && y_0 <= yy1 && yy0 <= y_1 && yy0 <= yy1) ||
          (y_0 >= y_1 && y_0 >= yy1 && yy0 >= y_1 && yy0 >= yy1)
         );
 }

/* Special malloc routine */
unsigned char *strmalloc(unsigned char *string, size_t size) {
  unsigned char *memory = malloc(size);
  return strcpy (memory, string);
 }

/* this converts an Allegro Keyboard Code into a standard ASCII code */
#define __KEYCODE_INLINE__ \
   if ((scancode >= 59 && scancode <= 68) || scancode == 87 || scancode == 88) \
      return 0x400 | scancode | (asciicode >> 8); \
   if (asciicode) return asciicode; else return 0x100 | scancode;

/* This converts an Allegro Keycode into a MagicTutu code */
int convert_keycode(int allegcode) {
  unsigned char scancode = (allegcode >> 8),
                asciicode = allegcode; /* No need for & 0xff */

  __KEYCODE_INLINE__
 }

/* This reads the keyboard, then return the MagicTutu code */
int read_convert_keyboard () {
  int allegcode = readkey ();
  unsigned char scancode = (allegcode >> 8),
                asciicode = allegcode; /* No need for & 0xff */

  __KEYCODE_INLINE__
 }

/* Get Window Drawable Width Area */
int window_h(int handle) {
   switch (windata[handle].wintype) {
    case WIN_NOBORDER: { return (windata[handle].flags & WF_INMINSTATE ? windata[handle].h_org : windata[handle].h); }
    case WIN_WITHFILE:
    case WIN_STANDARD: { return (windata[handle].flags & WF_INMINSTATE ? windata[handle].h_org : windata[handle].h) - 8; }
   }
  return -1; /* Error */
 }

/* Get Window Drawable Depth Area */
int window_v(handle) {
   switch (windata[handle].wintype) {
    case WIN_NOBORDER: { return (windata[handle].flags & WF_INMINSTATE ? windata[handle].v_org : windata[handle].v); }
    case WIN_STANDARD: { return (windata[handle].flags & WF_INMINSTATE ? windata[handle].v_org : windata[handle].v) - 15; }
    case WIN_WITHFILE: { return (windata[handle].flags & WF_INMINSTATE ? windata[handle].v_org : windata[handle].v) - 17 - biosfontheight; }
   }
  return -1; /* Error */
 }

/* Lock a window from use, plus all the windows that are parented to it */
void change_window_locks (bool lockstate, int handle, int dontlock) {
 int i, h;

  if (lockstate) { /* Check for the windows that needs locking */
   if (windata[handle].lock < 0) windata[handle].lock = handle;
  } else {
   if (windata[handle].lock == handle) windata[handle].lock = -1;
  }

   for (i = 0; i < wininpos; i++) {
    h = winhandles[i];
     if (h != dontlock && windata[h].parent == handle) { /* Is this window parented to handle */
      change_window_locks (lockstate, h, dontlock); /* Yes, also lock it, plus all its offsprings */
     }
   }
 }

/* Find an offspring of a window (-1 there is no offsprings) */
int window_offspring_of (int handle) {
  int i, h;
   for (i = 0; i < wininpos; i++) {
    h = winhandles[i];
    if (h != handle && windata[h].parent == handle) return h;
   }
  return -1;
 }

/* Find a window's handle with a specific title and parent */
int find_window_handle (char *title, int parent) {
  int i, z;

  if (!wininpos) return -1; /* Not found */
   if (parent < -1) {
    for (i = 0; i < wininpos; i++) {
     z = winhandles[i];
     if (strcmp(windata[z].title,title)==0) return z;
    }
   } else {
    for (i = 0; i < wininpos; i++) {
     z = winhandles[i];
      if (windata[z].parent == parent) {
       if (strcmp(windata[z].title,title)==0) return z;
      }
     }
   }
  return -1;
 }

/* Work out the size of the destined window */
void get_window_size (int type, int orgx, int orgy, int *szex, int *szey) {
   switch (type) { /* Check for desired window type */
    case WIN_NOBORDER: {
     *szex = orgx;
     *szey = orgy;
     break; }
    case WIN_STANDARD: {
     *szex = orgx + 8;
     *szey = orgy + 15;
     break; }
    case WIN_WITHFILE: {
     *szex = orgx + 8;
     *szey = orgy + 17 + biosfontheight;
     break; }
   }
 }

/* Get the screen relative coords of a point in the drawable area of window */
void get_screen_point (int handle, int x, int y, int *px, int *py) {
  switch (windata[handle].wintype) {
   case WIN_NOBORDER: {
    *px = window_x (handle) + x ; *py = window_y (handle) + y; break; }
   case WIN_STANDARD: {
    *px = window_x (handle) + 4 + x; *py = window_y (handle) + y + 11; break; }
   case WIN_WITHFILE: {
    *px = window_x (handle) + 4 + x; *py = window_y (handle) + y + 13 + biosfontheight; break; }
   }
 }

/* Allocate resources for a window */
int allocate_window (int posx, int posy, int szeh, int szev, int parent,
             int flags, int type, char *title, char *file,
             int (*redraw)(int handle, void *value),
             int (*close)(int handle, void *value),
             int (*special)(int handle,void *value),
             void *value) {
 int handle;

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: allocate_window is being executed at: %U\n", gui_timer);
#endif

   if (wininpos >= MAX_WINHANDLES) { /* Out of window handles */
#ifdef DEBUG_GUI
    fprintf(stderr, "gui.c: can not allocate any more handles at: %U\n", gui_timer);
#endif
    return -1; /* Return -1 */
   }

  handle = winhandles[wininpos];
  wininpos++;
#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: allocate_window chose to allocate handle: %U at: %U\n", handle, gui_timer);
  fprintf(stderr, "       ... now there is/are %U window(s) active.\n", wininpos);
#endif
  get_window_size(type, szeh, szev, &windata[handle].h, &windata[handle].v);
 /* Position it accordingly */
  if ((posx < 0) && (posx >= -10000)) posx = (scrn_widt - windata[handle].h) * -posx / 10000;
  if ((posy < 0) && (posy >= -10000)) posy = (scrn_hght - windata[handle].v) * -posy / 10000;
  windata[handle].x = posx;
  windata[handle].y = posy;
  windata[handle].title = title;
  windata[handle].filename = file;
  windata[handle].threadtime = windata[handle].timeleft = 0; /* Thread has not started */
  windata[handle].flags = flags;
  windata[handle].parent = parent;
  windata[handle].lock = -1; /* Lock is disabled */
  windata[handle].wintype = type;  /* Window Type */
  windata[handle].value = value; /* Value Offset */
   if ((windata[handle].bitmap = create_bitmap (windata[handle].h, windata[handle].v))==NULL) {
#ifdef DEBUG_GUI
    fprintf(stderr, "gui.c: allocate_window unable to allocate bitmap for new handle: %U at: %U\n", handle, gui_timer);
#endif
    wininpos--; return -1;
   }
  windata[handle].subbitmap = NULL;
  windata[handle].redraw = redraw;
  windata[handle].close = close;
  windata[handle].special = special;
  /* Set all special subroutines to off */
   windata[handle].interact = windata[handle].keyboard =
     windata[handle].minimize = windata[handle].maximize =
     windata[handle].active = windata[handle].inactive =
     windata[handle].thread = NULL;
   windata[handle].interface = NULL;
#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: allocate_window does window drawing for new handle: %U at: %U\n", handle, gui_timer);
#endif
  if (redraw (handle, value)) {
#ifdef DEBUG_GUI
   fprintf(stderr, "gui.c: allocate_window fails to draw new window for new handle: %U at: %U\n", handle, gui_timer);
#endif
   destroy_bitmap (windata[handle].bitmap);
   wininpos--; return -1;
  }
#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: allocate_window has allocated handle %U at: %U\n", handle, gui_timer);
#endif
  return handle;
 }

/* Enable / Disable inactive / active routines for window */
int win_activities (int handle, int (*active)(int handle, void *value),
                    int (*inactive)(int handle, void *value)) {
#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: win_activity is being executed for handle %U at: %U\n", handle, gui_timer);
#endif
  windata[handle].active = active;
  windata[handle].inactive = inactive;
  return 0;

 }

/* Enable/Disable Multithreading of a Window */
int win_multithread (int handle, int (*thread)(int handle, void *value), int rate) {

#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: win_multithread is being executed for handle %U at: %U\n", handle, gui_timer);
#endif

   if ((windata[handle].thread = thread) != NULL) {
    windata[handle].threadtime = rate;
    windata[handle].timeleft = gui_timer + rate;
#ifdef DEBUG_GUI
    fprintf(stderr, "gui.c: win_multithread set up threading for handle %U at: %U\n", handle, gui_timer);
#endif
   }
#ifdef DEBUG_GUI
   else {
    fprintf(stderr, "gui.c: win_multithread disabled threading for handle %U at: %U\n", handle, gui_timer);
   }
#endif
  return 0;
 }

/* Initialize the window hook */
int win_interact (int handle, int (*interact)(int handle, void *value)) {

#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: win_interact is being to executed for handle %U at: %U\n", handle, gui_timer);
#endif

  windata[handle].interact = interact;
  return 0;
 }

/* Initialize the special hook */
int win_special (int handle, int (*special)(int handle, void *value)) {

#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: win_special is being to executed for handle %U at: %U\n", handle, gui_timer);
#endif
  windata[handle].special = special;
  return 0;
 }

/* Can the drag controls be initialized, FALSE if drag controls can be initialized */
bool win_dragavail () {
  return drag_focus != -1;
 }

/* Start the drag controls */
bool win_dragstart (int handle, int (*control)(int handle, void *value),
                   int (*termin)(int handle, void *value)) {
  if (drag_focus == -1) { /* Able to install the hooks */
   drag_focus = handle;
   drag_control = control;
   drag_termin = termin;
   drag_flag = TRUE; /* Signal Drag */
   return FALSE;
  } else {
   return TRUE; /* Can not install the hooks */
  }
 }

/* End the drag controls */
bool win_dragend (int handle) {
   if (handle == drag_focus) { /* Correct Handle, end */
    drag_focus = -1;
    drag_control = drag_termin = NULL;
    return FALSE;
   }
  return TRUE; /* Wrong handle, can not end */
 }

/* Initialize the keyboard hook */
int win_keyboard (int handle, int (*keyboard)(int handle, void *value)) {

  windata[handle].keyboard = keyboard;
  return 0;
 }

/* Bring a window handle number out to the front */
bool bring_to_front (int handle) {
  int i, last;
  bool flag = TRUE;

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: bring_to_front is being executed for handle: %U at: %U\n", handle, gui_timer);
#endif

   if (!wininpos) return -1;
   i = last = (wininpos - 1);
   if (handle == winhandles[i]) return FALSE; /* Already at front */
    do {
     i--;
     if (i < 0) return TRUE; /* Not found */
     if (handle == winhandles[i]) flag = FALSE; /* Handle found */
    } while (flag);
   /* Now bring the handle to the front */
    for (; i < last; i++) {
     winhandles[i] = winhandles[i + 1];
    }
   winhandles[last] = handle;
  return FALSE;
 }

/* This procedure just draws the window border */
void draw_window_border (int handle) {
 int h = windata[handle].h, v = windata[handle].v, len, lmost,
     special_clr, primary_clr;
 BITMAP *bitmap = windata[handle].bitmap;

  switch (windata[handle].wintype) {
   case WIN_NOBORDER: { /* No Border */
    break; }
   case WIN_WITHFILE: {
      if (windata[handle].filename) {
       text_mode (-1);
       textout_centre (bitmap, biosfont, windata[handle].filename, h / 2, 12, clrsh_windwfile);
      }
     hline(bitmap, 4, 12 + biosfontheight, h - 5, clrsh_winfore);
    } /* Continues with WIN_STANDARD */
   case WIN_STANDARD: {
    primary_clr = (winactive == handle ? clrsh_winborder : clrsh_dwin_brda);
    special_clr = (winactive == handle ? clrsh_winbrdaii : clrsh_dwinbrdii);
    rect (bitmap, 0, 0,  h - 1, v - 1, special_clr);
    rect (bitmap, 1, 1,  h - 2, v - 2, primary_clr);
    putpixel (bitmap, 2, 2, primary_clr);
    putpixel (bitmap, h - 3, 2, primary_clr);
    hline(bitmap, 3, 2, h - 4, special_clr);
    rect (bitmap, 2, 3, h - 3, v - 3, primary_clr);
    hline(bitmap, 3, 4, h - 4, special_clr);
    hline(bitmap, 3, 5, h - 4, primary_clr);
    hline(bitmap, 3, 6, h - 4, special_clr);
    hline(bitmap, 3, 7, h - 4, primary_clr);
    hline(bitmap, 3, 8, h - 4, special_clr);
    hline(bitmap, 3, 9, h - 4, primary_clr);
    rect (bitmap, 3, 10, h - 4, v - 4, special_clr);
     if (windata[handle].title) {
      text_mode (-1);
      len = text_length (biosfont, windata[handle].title) + text_length (biosfont, "  ");
      lmost = (h - len) >> 1;
      rectfill(bitmap, lmost, 1, lmost + len - 1, text_height(biosfont) + 1, primary_clr);
      textout_centre(bitmap, biosfont, windata[handle].title, (h >> 1), 2,
       (winactive == handle ? clrsh_wintitle : clrsh_dwintitle));
     }
    /* Is the window closeable? */
     if (!(windata[handle].flags & WF_NOCLOSE) && windata[handle].close) {
      rect(bitmap, 3, 0, 13, 10, special_clr);
      rectfill(bitmap, 4, 1, 12, 9, clrsh_winbar_bg);
      rect(bitmap, 5, 2, 11, 8, clrsh_winbaricn);
      vline(bitmap, 6, 2, 8, clrsh_winbaricn);
      vline(bitmap, 10,2, 8, clrsh_winbaricn);
     }
    /* Special icon */
     if ((windata[handle].flags & WF_SPECIAL) && windata[handle].special) {
      rect(bitmap, 13, 0, 23, 10, special_clr);
      rectfill(bitmap,14,1,22,9,  clrsh_winbar_bg);
      hline(bitmap, 15,2,21, clrsh_winbaricn);
      rectfill(bitmap,15,3,16,4, clrsh_winbaricn);
      hline(bitmap, 15,5,21, clrsh_winbaricn);
      rectfill(bitmap,20,6,21,7,clrsh_winbaricn);
      hline(bitmap, 15,8,21, clrsh_winbaricn);
     }
    /* Minimize Icon */
     if (!(windata[handle].flags & WF_NOMINIMIZE)) {
      rect(bitmap, h - 14, 0, h - 4, 10, special_clr);
      rectfill(bitmap, h - 13, 1, h - 5, 9, clrsh_winbar_bg);
      rect(bitmap, h - 12, 2, h - 6, 3, clrsh_winbaricn);
      rect(bitmap, h - 11, 4, h - 7, 5, clrsh_winbaricn);
      rect(bitmap, h - 10, 6, h - 8, 7, clrsh_winbaricn);
      putpixel(bitmap, h - 9, 8, clrsh_winbaricn);
     }
    /* Maximize Icon */
    break; }
   } /* switch */
 }

/* This change the window appearance to denote activity / inactivity */
void show_activeness (int handle) {

  if (handle < 0) return; /* If handle is under zero */
  if ((windata[handle].flags & WF_INMINSTATE) && handle != winactive) return; /* Minimized window is inactive */
  if (prepare_to_draw_win(handle)) return; /* Prepare to draw window */
  draw_window_border(handle);  /* Redraw the window border */
  blit_window(handle);         /* Draw the window */
 }

/* This procedure deactives the current window */
bool deactive_window () {
 int former = winactive;

  /* if winactive is below zero */
  if (former < 0) return FALSE;
  /* Check if the current window can be Deactivated */
   if (windata[former].inactive) {
     if (windata[former].inactive (former, windata[former].value))
      return TRUE;
   }
  winlastactive = former; /* Maintain note about last active window */
  winactive = timer_dclick = -1; mouse_dclick = 0;
  show_activeness(former); /* Show inactivity with window */
  return FALSE;
 }

/* This reactivates a previously deactivated */
bool reactive_window () {
  if (winactive >= 0) return FALSE; /* If a window is already activated (stop) */

   if (windata[winlastactive].active) {
    if (windata[winlastactive].active (winlastactive, windata[winlastactive].value)) {
     return TRUE;
    }
   }
  /* Reactivation is a success */
  show_activeness (winactive = winlastactive);
  mouse_dclick = 0; timer_dclick = -1; /* Prevent double clicking across windows */
  return FALSE;
 }

/* This procedure activates a window, deactivating the previous one */
bool active_window (int handle) {
 int former = winactive;

  /* if winactive is not below zero */
  if (handle == former) {
   return FALSE; /* The window is already activated */
  } else if (former >= 0) {
   mouse_dclick = 0; timer_dclick = -1; /* Prevent double clicks across applets */
   /* Check if the current window can be Deactivated */
    if (windata[former].inactive) {
      if (windata[former].inactive (winactive, windata[former].value))
       return TRUE;
     }
   }

  /* Now activate the new window */
   if (windata[handle].active) {
    if (windata[handle].active (handle, windata[handle].value)) {
     /* Attempt Reactivation of the old window when failed to activate the new one */
      if (windata[former].active) {
        if (windata[former].active (former, windata[former].value)) {
          winlastactive = former; winactive = -1;
          show_activeness (former); /* Make it show inactiveness */
          return TRUE;
         }
       }
     return TRUE;
    }
   }

  winactive = handle;
  show_activeness (former);
  show_activeness (winactive);
  return FALSE;

 }

/* Prepare to draw onto a window */
bool prepare_to_draw_win (int handle) {
 int cx, num;

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: prepare_to_draw_win is being executed for handle: %U at: %U\n", handle, gui_timer);
#endif

   if (windata[handle].flags & WF_INMINSTATE) {
    if (restore_window (handle)) return TRUE; /* Restore Window */
   }
  if (windata[handle].y < 0 || !windata[handle].bitmap) return TRUE; /* Can not draw */

   if (!(wininpos > 0 && winhandles[wininpos-1] == handle)) { /* Is it at the top */
    /* Check if it is not cluttered behind something */
     for (cx = wininpos - 1; ; cx--) {
      if (cx < 0) return TRUE;
      if ((num = winhandles[cx]) == handle) return FALSE;
       if (check_for_overlap(windata[num].x, windata[num].y,
                             windata[num].h, windata[num].v,
                             windata[handle].x, windata[handle].y,
                             windata[handle].h, windata[handle].v)) break;
     }
    /* If the window is cluttered behing something, bring it to the front */
     if (bring_to_front(handle)) return TRUE;
     scare_mouse ();
     temporary_hide_tbmos ();
     blit (windata[handle].bitmap, screen, 0, 0,
           windata[handle].x, windata[handle].y,
           windata[handle].h, windata[handle].v);
     redraw_all_tbmos (); /* Redraw the TBMOs */
     unscare_mouse ();
    }
  return FALSE; /* Success, now the contents can be drawn */
 }

/* Blit window bitmap onto screen, do not push forward */
void blit_window (int handle) {

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: blit_window is being executed for handle: %U at: %U\n", handle, gui_timer);
#endif

  /* If the BITMAP is a sub-bitmap */
   if (!windata[handle].bitmap || windata[handle].y < 0) return; /* Can not blit an empty bitmap */
  scare_mouse ();
  temporary_hide_tbmos (); /* Temporary hide the TBMOs */
  blit (windata[handle].bitmap, screen, 0, 0,
        windata[handle].x, windata[handle].y,
        windata[handle].h, windata[handle].v);
  redraw_all_tbmos (); /* Redraw the TBMOs */
  unscare_mouse ();
  return;
 }

/* Blit a sub section of a window */
void blit_sub_window (int handle, int x, int y, int h, int v) {

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: blit_sub_window is being executed for handle: %U at: %U\n", handle, gui_timer);
#endif

  /* If the BITMAP is a sub-bitmap */
  scare_mouse ();
   switch (windata[handle].wintype) {
    case WIN_NOBORDER: {
     blit (windata[handle].bitmap, screen, x, y,
           windata[handle].x + x, windata[handle].y + y, h, v);
     break; }
    case WIN_STANDARD: {
     blit (windata[handle].subbitmap, screen, x, y,
           windata[handle].x + x + 4, windata[handle].y + y + 11, h, v);
     break; }
    case WIN_WITHFILE: {
     blit (windata[handle].subbitmap, screen, x, y,
           windata[handle].x + x + 4,
           windata[handle].y + y + biosfontheight + 13, h, v);
     break; }
   }
  unscare_mouse ();
  return;
 }

/* Get the mouse cursor location, relative to window position
 * TRUE:  Cursor out of range (or window is invaild)
 * FALSE: Cursor is within range
 */
bool win_get_mouse (int handle, int *x, int *y) {

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: win_get_mouse is being executed for handle: %U at %U\n", handle, gui_timer);
#endif

  if (windata[handle].y < 0) return TRUE; /* The window is off screen */
   switch (windata[handle].wintype) {
    case WIN_NOBORDER:
            { if ((*x = int_mouse_x - windata[handle].x) >= windata[handle].h) return TRUE;
              if ((*y = int_mouse_y - windata[handle].y) >= windata[handle].v) return TRUE;
              break; }
    case WIN_STANDARD:
            { if ((*x = int_mouse_x - windata[handle].x - 4) >= (windata[handle].h - 8)) return TRUE;
              if ((*y = int_mouse_y - windata[handle].y - 11)>= (windata[handle].v - 15)) return TRUE;
              break; }
    case WIN_WITHFILE:
            { if ((*x = int_mouse_x - windata[handle].x - 4) >= (windata[handle].h - 8)) return TRUE;
              if ((*y = int_mouse_y - windata[handle].y - 13 - biosfontheight)>= (windata[handle].v - 15)) return TRUE;
              break; }
    default: { return TRUE; }
   }
  if (*x < 0 || *y < 0) return TRUE;
  return FALSE;
 }

/* Place the mouse cursor onto the screen */
void win_put_mouse (int handle, int x, int y) {

  if (windata[handle].y < 0) return; /* Do not move the mouse */
   switch (windata[handle].wintype) {
    case WIN_NOBORDER:
            { position_mouse (windata[handle].x + x, windata[handle].y + y); break; }
    case WIN_STANDARD:
            { position_mouse (windata[handle].x + 4 + x, windata[handle].y + 11 + y); break; }
    case WIN_WITHFILE:
            { position_mouse (windata[handle].x + 4 + x, windata[handle].y + 13 + biosfontheight); break; }
   }
 }

/* Just draws the window outline, that is it */
bool draw_window (int handle) {
 int h, v;
 BITMAP *bitmap;

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: draw_window is being executed for handle: %U at: %U\n", handle, gui_timer);
 fprintf(stderr, "       ... with title \"%s\" \n", windata[handle].title);
#endif

  temporary_hide_tbmos (); /* Temporary hide the TBMOs */
   if (windata[handle].flags & WF_INMINSTATE) {
    if (restore_window_on_redraw (handle)) return TRUE;
   }
  if (windata[handle].y < 0) return TRUE; /* Can not draw on a hidden window */
  h = windata[handle].h; v = windata[handle].v; bitmap = windata[handle].bitmap;

   if (winhandles[wininpos - 1] != handle) { /* Is it at the top */
    bring_to_front (handle);
   }
  if (!bitmap) return;
  switch (windata[handle].wintype) {
   case WIN_NOBORDER: { /* Completely unbordered */
     if (!windata[handle].subbitmap) {
      windata[handle].subbitmap = create_sub_bitmap(bitmap, 0, 0, h, v);
     }
    break; }
   case WIN_STANDARD: {
     if (!windata[handle].subbitmap) {
      windata[handle].subbitmap = create_sub_bitmap(bitmap, 4, 11, h - 8, v - 15);
     }
    break; }
   case WIN_WITHFILE: {
     if (!windata[handle].subbitmap) {
      windata[handle].subbitmap = create_sub_bitmap(bitmap, 4,
         13 + biosfontheight, h - 8, v - 17 - biosfontheight);
     }
    break; }
   } /* switch */
  clear_to_color (bitmap, clrsh_winback);
  draw_window_border (handle);
  return FALSE;
 }

/* Complete Screen Refresh -- This routine completely refreshes the screen */
void refresh_screen_completely () {
  int i, z;
 /* First clear the screen */

#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: refresh_screen_completely is being executed at: %U\n", gui_timer);
#endif

  scare_mouse ();
  temporary_hide_tbmos ();
   if (wallpaper) {
    blit (wallpaper, screen, 0, 0, 0, 0, scrn_widt, scrn_hght);
   } else {
    clear_to_color (screen, clrsh_wallpaper);
   }
   if (wininpos) {
   /* Blit all sprites onto the screen */
    for (i = 0; i < wininpos; i++) {
      if (windata[(z = winhandles[i])].y >= 0 && windata[z].bitmap) {
        blit (windata[z].bitmap, screen, 0, 0,
              windata[z].x, windata[z].y, windata[z].h, windata[z].v);
      }
    }
   }

  redraw_all_tbmos (); /* Redraw the TBMOs */
  unscare_mouse ();
 }

/* This procedure refresh an area of the screen */
void refresh_screen_area (int x, int y, int h, int v) {
 int i, z, cl, cr, ct, cb;

  scare_mouse ();
  temporary_hide_tbmos ();

 /* Paint over a section */
   if (wallpaper) {
    blit (wallpaper, screen, x, y, x, y, h, v);
   } else {
    rectfill(screen, x, y, x + h - 1, y + v - 1, clrsh_wallpaper);
   }

  cl = screen->cl; ct = screen->ct;
  screen->cl = x; screen->ct = y;
  cr = screen->cr; cb = screen->cb;
  screen->cr = x + h; screen->cb = y + v;

   for (i = 0; i < wininpos; i++) {
    if (windata[(z = winhandles[i])].y >= 0 && windata[z].bitmap) {
     if (check_for_overlap (x, y, h, v, windata[z].x, windata[z].y,
                                        windata[z].h, windata[z].v)) {
        blit (windata[z].bitmap, screen, 0, 0,
              windata[z].x, windata[z].y, windata[z].h, windata[z].v);
      } /* if */
     } /* if */
    } /* for */

  screen->cl = cl; screen->cr = cr; screen->ct = ct; screen->cb = cb;
  redraw_all_tbmos (); /* Redraw the TBMOs */
  unscare_mouse ();

 }

/* This procedure destroys a window resource */
void destroy_window (int handle) {
#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: destroy_window is being executed for handle: %U at: %U\n", handle, gui_timer);
#endif
  if (bring_to_front (handle)) { /* Bring the handle to the front */
    return; /* The resource is already destroyed */
   }
  wininpos--;
   if (winactive == handle) {
     winactive = -1; /* No need to "deactivate" the destroyed window */
     active_window (winhandles[wininpos - 1]); /* Set the new active handle */
    }
#ifdef DEBUG_GUI
  fprintf(stderr, "gui.c: destroy_window has disposed handle %U at: %U\n", handle, gui_timer);
#endif
  /* Destroy drag controls */
   if (handle == drag_focus) {
    drag_focus = -1;
    drag_control = drag_termin = NULL;
   }
  /* Destroy its temporary bitmap overlays */
  /* Destroy its main resource */
  destroy_bitmap (windata[handle].subbitmap);
  destroy_bitmap (windata[handle].bitmap);
  refresh_screen_area (windata[handle].x, windata[handle].y,
                       windata[handle].h, windata[handle].v);
 }

/* This formally closes down a window resource, It will ask the window's
   own close down routine to do the disposal that it requires, before
   destroying the window resource.
 */
int close_window (int handle) {

  if (!(windata[handle].flags & WF_CLSNOREACT) && handle != winactive)
   /* Must reactive window before closing (unless if WF_CLSNOREACT is set) */
   if (active_window(handle)) return -1;
  if (windata[handle].close) {
   if (windata[handle].close (handle, windata[handle].value)) {
    return -1; /* Window can not be closed */
   } else {
    destroy_window (handle); /* Destroy the window resource */
    return 0; /* Window has closed down formally */
   }
  } else
   return -1; /* No close down handle, means no close down by this routine */
 }

/* Similar to close_window, but for the special icon */
int call_window_special (int handle) {
  if (!(windata[handle].flags & WF_SPCNOREACT) && handle != winactive)
   /* Must reactive window before closing (unless if WF_CLSNOREACT is set) */
   if (active_window(handle)) return -1;
  if (windata[handle].special) {
   if (windata[handle].special (handle, windata[handle].value)) {
    return -1; /* The special refuses the close down */
   } else {
    destroy_window (handle); /* Destroy the window resource */
    return 0; /* The special has executed a close down */
   }
  } else
   return -1; /* The special is not around */
 }

/* This procedure closes down a window with all its offsprings */
int close_window_with_offsprings (int handle) {
 int i;
  /* Close down all its offsprings */
   while ((i = window_offspring_of (handle)) >= 0) {
    if (close_window_with_offsprings (i)) return -1; /* Can not close down correctly */
   }
  return close_window(i);
 }

/* Redraw a specified window
 * handle: window's handle
 * wakeup: TRUE if the window is allowed to be maximized
 */
bool win_redraw (int handle, bool wakeup) {
   if (!wakeup && (windata[handle].y < 0 || (windata[handle].flags & WF_INMINSTATE))) {
    return TRUE;
   }

  return (windata[handle].redraw (handle, windata[handle].value) != 0);
 }

/* GUI screen capture routine */
void gui_screen_capture () {
  FILE *file;
  BITMAP *bitmap;
  char filename [12];

  /* Make sure that Print screen is released */
   while (key[KEY_PRTSCR]) { } /* screen capture */

  /* Too Many Screen Captures */
   if (screen_capture_counts == 100000) {
    return;
   }
  bitmap = create_bitmap(scrn_widt, scrn_hght);
   if (bitmap) { /* If there is a bitmap */
    scare_mouse ();
    blit(screen,bitmap, 0, 0, 0, 0, scrn_widt, scrn_hght);
     while (screen_capture_counts < 100000) { /* Do not overwrite existing files */
      sprintf(filename,"gui%U.pcx", screen_capture_counts);
       if ((file = fopen(filename, "rb")) == NULL) { /* Save if the file can not be opened */
        save_bitmap (filename, bitmap, set_gui_palette);
        screen_capture_counts++;
        break;
       } else { /* Advance to the next number */
        fclose (file);
        screen_capture_counts++;
       }
      }
    destroy_bitmap (bitmap);
    unscare_mouse ();
   }
 }

/* GUI mouse interact routine */
void gui_mouse_interact () {
 int z, h, v, x, y,
        handle = (winactive < 0 ? winhandles[wininpos - 1] : winactive);
 bool flag = FALSE, virgin = TRUE;

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: gui_mouse_interact is being executed at: %U\n", gui_timer);
#endif

 window_relocate = FALSE;
 for (;;) {
/* If within the active window (can not use a locked window) */
     if (windata[handle].y < 0) {
      x = -1; /* Offscreen Window */
     } else {
      x = int_mouse_x - windata[handle].x; h = windata[handle].h;
      y = int_mouse_y - windata[handle].y; v = windata[handle].v;
     }
     if (x >= 0 && x < h && y >=0 && y < v) {
      flag = TRUE;
       if (windata[handle].lock < 0 && !(windata[handle].flags & WF_DISABLED)) {
        switch (windata[handle].wintype) {
         case WIN_NOBORDER: {
           if (!active_window(handle)) {
            if (windata[handle].interact)
             windata[handle].interact (handle, windata[handle].value);
           }
          break; }
         case WIN_WITHFILE:
         case WIN_STANDARD: { /* The Standard Window */
           if (x >= 4 && x < (h - 4) && y >= 11 && y < (v - 4)) {
            if (!active_window(handle)) {
              if (windata[handle].interact) {
                if (windata[handle].wintype != WIN_WITHFILE || y > (13 + biosfontheight))
                 windata[handle].interact (handle, windata[handle].value);
               }
             }
           } else if (!(windata[handle].flags & WF_NOCLOSE) &&
                      windata[handle].close && x>=4 && y > 0 && y < 10 && x <= 12) {
            close_window (handle);
           } else if ((windata[handle].flags & WF_SPECIAL) &&
                      windata[handle].special && x>=14 & y > 0 & y < 10 && x <= 22) {
            call_window_special (handle);
           } else if (!(windata[handle].flags & WF_NOMINIMIZE) &&
                       x>=(h - 13) & y > 0 & y < 10 & x < (h - 4)) { /* Minimize Routine */
            rollup_window (handle); /* Actually a roll up at the moment */
           } else if (x >= 4 && x <= (h - 4) && y < 11) {
             if (windata[handle].flags & WF_NORELOCATE)
              active_window(handle);
             else
              if (!active_window(handle)) window_relocate = TRUE;
           } else
             active_window(handle);
          break; }
         } /* switch */
       } /* if windata[handle].lock */
     } /* if (within the window area) */
   if (flag) break;
    if (virgin) { /* Virgin Call (at the first time) */
     virgin = FALSE;
     z = wininpos - 1;
     if (handle == winhandles [z]) { z--; }
     if (z < 0) break; else handle = winhandles[z];
    } else {
     if ( z == 0) break; else handle = winhandles[z -= 1];
    }
  }
 }

/* Bitmap for relocation animation */
int reloc_x, reloc_y, reloc_h, reloc_v, reloc_hand;
BITMAP *reloc_a, *reloc_b, *reloc_c, *reloc_d;

/* Update window relocation */
void gui_update_window_relocate () {
  scare_mouse ();
   if (reloc_a) {
    blit(reloc_a, screen, 0, 0, reloc_x, reloc_y, reloc_h, 4);
    blit(reloc_b, screen, 0, 0, reloc_x, reloc_y + reloc_v - 4, reloc_h, 4);
    blit(reloc_c, screen, 0, 0, reloc_x, reloc_y + 4, 4, reloc_v - 8);
    blit(reloc_d, screen, 0, 0, reloc_x + reloc_h - 4, reloc_y + 4, 4, reloc_v - 8);
   } else {
    reloc_a = create_bitmap (reloc_h, 4);
    reloc_b = create_bitmap (reloc_h, 4);
    reloc_c = create_bitmap (4, reloc_v - 4);
    reloc_d = create_bitmap (4, reloc_v - 4);
   }
  reloc_x = windata[reloc_hand].x + int_mouse_x - org_mouse_x;
  reloc_y = windata[reloc_hand].y + int_mouse_y - org_mouse_y;
  /* Check if x is in range of screen */
   if (reloc_x < 0) {
    reloc_x = 0;
   } else {
    if (reloc_x > (scrn_widt - reloc_h)) reloc_x = scrn_widt - reloc_h;
   }
  /* Check if y is in range of screen */
   if (reloc_y < 0) {
    reloc_y = 0;
   } else {
    if (reloc_y > (scrn_hght - reloc_v)) reloc_y = scrn_hght - reloc_v;
   }
  /* blit the screen to temporary bitmap */
  blit(screen, reloc_a, reloc_x, reloc_y, 0, 0, reloc_h, 4);
  blit(screen, reloc_b, reloc_x, reloc_y + reloc_v - 4, 0, 0, reloc_h, 4);
  blit(screen, reloc_c, reloc_x, reloc_y + 4, 0, 0, 4, reloc_v - 8);
  blit(screen, reloc_d, reloc_x + reloc_h - 4, reloc_y + 4, 0, 0, 4, reloc_v - 8);

  rect(screen, reloc_x, reloc_y, reloc_x + reloc_h - 4, reloc_y + reloc_v - 4, clr_black);
  rect(screen, reloc_x + 1, reloc_y + 1, reloc_x + reloc_h - 3, reloc_y + reloc_v - 3, clr_d_grey);
  rect(screen, reloc_x + 2, reloc_y + 2, reloc_x + reloc_h - 2, reloc_y + reloc_v - 2, clr_l_grey);
  rect(screen, reloc_x + 3, reloc_y + 3, reloc_x + reloc_h - 1, reloc_y + reloc_v - 1, clr_white);
  unscare_mouse ();
 }

/* Being window relocation */
void gui_being_window_relocate (int handle) {
  reloc_hand = handle;
  reloc_a = reloc_b = reloc_c = reloc_d = NULL;
  reloc_x = windata[reloc_hand].x; reloc_h = windata[reloc_hand].h;
  reloc_y = windata[reloc_hand].y; reloc_v = windata[reloc_hand].v;
  gui_update_window_relocate ();
 }

/*End window relocation */
void gui_end_window_relocate () {
 int orig_x = windata[reloc_hand].x, orig_y = windata[reloc_hand].y;

   if (reloc_a) {
    scare_mouse ();
    blit(reloc_a, screen, 0, 0, reloc_x, reloc_y, reloc_h, 4);
    blit(reloc_b, screen, 0, 0, reloc_x, reloc_y + reloc_v - 4, reloc_h, 4);
    blit(reloc_c, screen, 0, 0, reloc_x, reloc_y + 4, 4, reloc_v - 8);
    blit(reloc_d, screen, 0, 0, reloc_x + reloc_h - 4, reloc_y + 4, 4, reloc_v - 8);
    destroy_bitmap (reloc_a);
    destroy_bitmap (reloc_b);
    destroy_bitmap (reloc_c);
    destroy_bitmap (reloc_d);
    unscare_mouse ();
   }

  windata[reloc_hand].x = reloc_x;
  windata[reloc_hand].y = reloc_y;
  refresh_screen_area (orig_x, orig_y, reloc_h, reloc_v);
  blit_window (reloc_hand);
 }

/* The GUI kernal -- This handles multithreaded windows */
void gui_kernal () {
 int i = 0, handle, upd_mouse_x, upd_mouse_y;
 bool movement, drag_lock;

#ifdef DEBUG_GUI
 fprintf(stderr, "gui.c: gui_kernal is being executed at: %U\n", gui_timer);
#endif

  show_mouse (screen);
  position_mouse (scrn_widt >> 1, scrn_hght >> 1);
   while (wininpos) {
#ifdef DEBUG_ESCAPE
    if (key[KEY_ESC]) exit(255); /* Ye PANIC key */
#endif
    if (key[KEY_PRTSCR]) gui_screen_capture (); /* screen capture */
    /* Check the Mouse */
    upd_mouse_x = mouse_x;
    upd_mouse_y = mouse_y;
    movement = ((int_mouse_x != upd_mouse_x) || (int_mouse_y != upd_mouse_y));
    int_mouse_x = upd_mouse_x;
    int_mouse_y = upd_mouse_y;
     if ((mouse_status = mouse_b)) {
      if (click_guard) {
        if (timer_dclick < 0) {
         timer_dclick = dclick_time; mouse_dclick = 0; mouse_first_state = mouse_status;
#if defined DEBUG_GUI || defined DEBUG_GUI_MOUSE
         fprintf(stderr, "gui.c: gui_kernal has detected a click at: %U\n", gui_timer);
#endif
        } else {
         /* Check if the mouse cursor has not strayed too far during double clicking */
          if (ABS(org_mouse_x - int_mouse_x) >= dclick_maxstray ||
              ABS(org_mouse_y - int_mouse_y) >= dclick_maxstray) {
           mouse_dclick = 0;
          } else {
           mouse_dclick = mouse_first_state & mouse_status;
          }
#if defined DEBUG_GUI || defined DEBUG_GUI_MOUSE
          if (mouse_dclick) {
           fprintf(stderr, "gui.c: gui_kernal has detected a double click within %U units at: %U\n", timer_dclick, gui_timer);
          } else {
           fprintf(stderr, "gui.c: gui_kernal has detected a failed double click within %U units at: %U\n", timer_dclick, gui_timer);
          }
#endif
         timer_dclick = -1;
        }
       org_mouse_x = int_mouse_x;
       org_mouse_y = int_mouse_y;
       click_guard = FALSE;
       gui_mouse_interact ();
#ifdef DEBUG_GUI
       fprintf(stderr, "gui.c: gui_kernal has finished with gui_mouse_interact at: %U\n", gui_timer);
#endif
       if (wininpos <= 0) break;
      } else { /* Check for Drag */
       if (!drag_flag && (ABS(org_mouse_x - int_mouse_x) > drag_setting ||
                          ABS(org_mouse_y - int_mouse_y) > drag_setting)) {
        if (window_relocate) { gui_being_window_relocate (winactive); drag_lock = TRUE; }
        drag_flag = TRUE;
        timer_dclick = -1; /* Do not register a double click in a drag */
       } else if (drag_lock && movement) { /* Execute the drag */
        if (window_relocate) gui_update_window_relocate ();
       } else if (drag_flag && drag_control) { /* Perform the drag control */
        if (drag_control(drag_focus, windata[drag_focus].value)) drag_flag = FALSE;
       }
      }
     } else {
       if (drag_flag) { /* Check for the drag flag */
         if (drag_lock && window_relocate) { /* During a window relocate */
          gui_end_window_relocate ();
         } else if (drag_termin) { /* Terminate the drag */
          drag_termin(drag_focus, windata[drag_focus].value);
         }
        drag_flag = drag_lock = FALSE;
       }
      click_guard = (!mouse_status);
     }

    if (drag_lock) continue;
    if (wininpos <= 0) break;
     if (winactive >=0 && keypressed () && windata[winactive].keyboard != NULL) { /* Has a key been pressed */
      /* Service the keyboard request by its routine */
       windata[winactive].keyboard (winactive, windata[winactive].value);
     } else { /* No keyboard service */
      clear_keybuf ();
     }

   /* Perform a thread */
    if (wininpos <= 0) break; else if (i >= wininpos) i = 0;
    handle = winhandles [i]; /* Get handle */
     if (windata[handle].thread != NULL) { /* Is this thread serviceable? */
      if (gui_timer >= windata[handle].timeleft) {
      /* Service Window's thread request */
#ifdef DEBUG_GUI
       fprintf(stderr, "gui.c: gui_kernal is executing thread for handle: %U at: %U\n", handle, gui_timer);
#endif
       windata[handle].thread (handle, windata[handle].value);
      /* Do not overserve it! */
        if ((windata[handle].timeleft += windata[handle].threadtime) < gui_timer) {
         windata[handle].timeleft = gui_timer + windata[handle].threadtime;
#ifdef DEBUG_GUI
         fprintf(stderr, "gui.c: gui_kernal has detected too much overhead at: %U\n", gui_timer);
#endif
        }
      }
     }
    i++; /* Advance to the next handle */
   }
 }

/* Ideas for the GUI icons:

   123456789   123456789
  1.........  1.........
  2....*....  2.*******.
  3...***...  3.*******.
  4...***...  4..*****..
  5..*****..  5..*****..
  6..*****..  6...***...
  7.*******.  7...***...
  8.*******.  8....*....
  9.........  9.........

 */

