/*
 * GUI routine header files
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU LGPL version 2.
 *
 * See the file "COPYING.LIB" for software license details.
 */

#ifndef __GSW_GUI_H__
#define __GSW_GUI_H__
#include "guigrlib.h"
#include <string.h>

#define MAX_WINHANDLES 1024 /* Maximum of 1024 handles for windows */

 #ifndef bool
  #define bool char
 #endif
 #ifndef TRUE
  #define TRUE (!0)
  #define FALSE 0
 #endif

/* The BIOS font */
extern FONT *biosfont;
extern int biosfontheight;

/* Mouse Button Status */
extern int mouse_status, mouse_dclick, int_mouse_x, mouse_type,
           int_mouse_y, org_mouse_x, org_mouse_y, mouse_first_state;
extern int drag_setting, dclick_time, timer_dclick, dclick_maxstray;
extern bool click_guard, drag_flag;

/* Screen dimensions */
extern int scrn_cntr, /* Centre of Screen */
           scrn_widt, /* Screen Width */
           scrn_hght, /* Screen Height */
           clr_mode;  /* Colour Mode */
extern long long gui_timer; /* GUI timer */

/* The wallpaper */
extern BITMAP *wallpaper;

/* The current GUI palette */
extern RGB *set_gui_palette;

/* Window type flags */
#define WIN_NOBORDER   0
#define WIN_STANDARD   1
#define WIN_WITHFILE   2

/* Window flags */
#define WF_NO_OPTIONS   0x00000000 /* Just to make the code look neater */
#define WF_RISE         0x00000001 /* Can this window rise with mouse cursor */
#define WF_NORELOCATE   0x00000002 /* No relocation of window? */
#define WF_NOCLOSE      0x00000004 /* Disable closing the window by user? */
#define WF_DISABLED     0x00000008 /* Is the Window disabled for use? */
#define WF_CLSNOREACT   0x00000010 /* No Reactivation is required when closing */
#define WF_SPECIAL      0x00000020 /* Special icon is available */
#define WF_SPCNOREACT   0x00000040 /* Do not active on the special icon */
#define WF_NOMINIMIZE   0x00000080 /* Do not allow minimization */
/* Special State Flags */
#define WF_INMINSTATE   0x00010000 /* In Minimize State */

/* Window structure */
 typedef struct {
    int      x,          /* x position on screen */
             y,          /* y position on screen */
             h,          /* width */
             v,          /* height */
             x_org,      /* Original X positon */
             y_org,      /* Original Y position */
             h_org,      /* Original H size */
             v_org;      /* Original V size */
unsigned int flags,      /* The flags that denote window behavior */
             threadtime; /* Thread timing rate */
long long    timeleft;      /* Amount of waiting before calling thread again (64-bit integer) */
   int       parent,        /* The window's parent */
             wintype;       /* Type of Window (for GUI interacting routines) */
   char      *title,        /* Window title */
             *filename;     /* Window filename (subheader) */
   int       lock;          /* Specifies the locker of this window */
   void      *value;        /* Applet data that must be passed to the handlers */
   BITMAP    *bitmap,       /* Bitmap that contains the window image */
             *subbitmap;    /* Writeable area of the window */
   int (*redraw)   (int handle, void *value);  /* Redraw window - Initialize */
   int (*close)    (int handle, void *value);  /* Close window */
   int (*interact) (int handle, void *value);  /* Interact window */
   int (*keyboard) (int handle, void *value);  /* Keyboard control */
   int (*minimize) (int handle, void *value);  /* Minimize window */
   int (*maximize) (int handle, void *value);  /* Maximize window */
   int (*active)   (int handle, void *value);  /* Activate window */
   int (*inactive) (int handle, void *value);  /* Deactivate window */
   int (*thread)   (int handle, void *value);  /* Window's Task Threading */
   int (*special)  (int handle, void *value);  /* Special Icon Task */
   int (*interface)(int handle, int cmd, void *value, void *pass); /* Interface with window -- For one window to talk with another */
  } WINSTRUCT;

/* Window Data Arrays */
 extern WINSTRUCT windata [MAX_WINHANDLES]; /* Window Information */
 extern int winhandles [MAX_WINHANDLES];    /* List of handles that are in use */
 extern int wininpos;                       /* Handles are in use */
 extern int winactive, winlastactive;       /* Active Window (Last active window) */

/* Standard Palette Colours */
#define clr_maxno   18
extern char *colornames [];
extern RGB  rgbcolors [];
extern int pal_stndrd [];
#define clr_black   pal_stndrd [0]
#define clr_d_blue  pal_stndrd [1]
#define clr_d_green pal_stndrd [2]
#define clr_d_cyan  pal_stndrd [3]
#define clr_red     pal_stndrd [4]
#define clr_purple  pal_stndrd [5]
#define clr_brown   pal_stndrd [6]
#define clr_l_grey  pal_stndrd [7]
#define clr_d_grey  pal_stndrd [8]
#define clr_blue    pal_stndrd [9]
#define clr_green   pal_stndrd [10]
#define clr_cyan    pal_stndrd [11]
#define clr_pink    pal_stndrd [12]
#define clr_magenta pal_stndrd [13]
#define clr_yellow  pal_stndrd [14]
#define clr_white   pal_stndrd [15]
#define clr_orange  pal_stndrd [16]
#define clr_olive   pal_stndrd [17]

/* GUI colour palette */
#define clrsh_maxno     38
extern char *clrschemenames [];
extern int colorscheme [];
extern RGB rgbscheme [];
#define clrsh_wallpaper colorscheme[0]
#define clrsh_winback   colorscheme[1]
#define clrsh_winfore   colorscheme[2]
#define clrsh_guioutlin colorscheme[3]
#define clrsh_guibutton colorscheme[4]
#define clrsh_guibuttxt colorscheme[5]
#define clrsh_winborder colorscheme[6]
#define clrsh_wintitle  colorscheme[7]
#define clrsh_winbrdaii colorscheme[8]
#define clrsh_guiotlnii colorscheme[9]
#define clrsh_winbaricn colorscheme[10]
#define clrsh_winbar_bg colorscheme[11]
#define clrsh_dwin_brda colorscheme[12]
#define clrsh_dwintitle colorscheme[13]
#define clrsh_dwinbrdii colorscheme[14]
#define clrsh_fguiotlin colorscheme[15]
#define clrsh_fguibtbod colorscheme[16]
#define clrsh_fguibttxt colorscheme[17]
#define clrsh_fguiotlii colorscheme[18]
#define clrsh_guichek_o colorscheme[19]
#define clrsh_guichek_b colorscheme[20]
#define clrsh_guichek_m colorscheme[21]
#define clrsh_fguicheko colorscheme[22]
#define clrsh_fguichekb colorscheme[23]
#define clrsh_fguichekm colorscheme[24]
#define clrsh_guiradi_o colorscheme[25]
#define clrsh_guiradi_b colorscheme[26]
#define clrsh_guiradi_m colorscheme[27]
#define clrsh_fguiradio colorscheme[28]
#define clrsh_fguiradib colorscheme[29]
#define clrsh_fguiradim colorscheme[30]
#define clrsh_guitext_o colorscheme[31]
#define clrsh_guitext_b colorscheme[32]
#define clrsh_guitext_e colorscheme[33]
#define clrsh_fguitexto colorscheme[34]
#define clrsh_fguitextb colorscheme[35]
#define clrsh_fguitexte colorscheme[36]
#define clrsh_windwfile colorscheme[37]

/* The RGB MAP for the colours */
extern RGB_MAP clr_conv_map;

void make_rgb_map (RGB *pal); /* Setup the 8-bit palette */
int initialize_gui ();        /* Initialize GUI */
unsigned char *strmalloc(unsigned char *string, size_t size); /* Special malloc routine */
bool win_get_mouse (int handle, int *x, int *y); /* Win Get Mouse */
void win_put_mouse (int handle, int x, int y); /* Position the Mouse */
int win_multithread (int handle, int (*thread)(int handle, void *value), int rate); /* Multithread */
int win_keyboard (int handle, int (*keyboard)(int handle, void *value)); /* Keyboard */
int win_activities (int handle, int (*active)(int handle, void *value),
                    int (*inactive)(int handle, void *value)); /* Active / Disactive changes */
void change_window_locks (bool lockstate, int handle, int dontlock); /* Lock a window, plus all its offsprings */
bool win_redraw (int handle, bool wakeup); /* redraws a window */
void refresh_screen_completely (); /* Refreshes the screen completely */
void destroy_window (int handle); /* Destroys a window resource */
int close_window (int handle); /* Formally closes down a window resource */
int close_window_with_offsprings (int handle); /* Formally closes down a window plus all its offsprings */
int call_window_special (int handle); /* Call the window special */
void get_window_size (int type, int orgx, int orgy, int *szex, int *szey); /* Get window size */
void get_screen_point (int handle, int x, int y, int *px, int *py); /* Get screen point of a point in window */
int allocate_window (int posx, int posy, int szeh, int szev, int parent,
               int flags, int type, char *title, char *file,
               int (*redraw)(int handle, void *value),
               int (*close)(int handle, void *value),
               int (*special)(int handle,void *value),
               void *value);
int convert_keycode(int allegcode); /* Make keyboard reading more presentable */
int read_convert_keyboard (); /* Same as convert_keycode, but reads the keyboard as well */
bool deactive_window (); /* Deactive the current window */
bool active_window (int handle); /* Actives a window */
bool reactive_window (); /* Reactive a previously deactived window */
void blit_window (int handle); /* blit window onto screen */
void blit_sub_window (int handle, int x, int y, int h, int v); /* blit sub section of window */
bool prepare_to_draw_win (int handle); /* prepare to draw on window */
int window_h(int handle); /* Window Drawable Width */
int window_v(int handle); /* Window Drawable Depth */
int find_window_handle (char *title, int parent); /* Find window's handle */
void gui_kernal ();
void register_single_click (); /* Register single click without going to double click phase */
bool draw_window (int handle);
bool win_dragstart (int handle, int (*control)(int handle, void *value),
                   int (*termin)(int handle, void *value)); /* Initalize a drag control */
bool win_dragend (int handle); /* Conclude a drag control */
bool win_dragavail (); /* Can the drag controls be initialized, TRUE if the case */

/* magic string replacer */
static inline string_replace (char **var, char *str) {
  if (*var) free (*var);
  *var = strdup(str);
 }

/* Define some macros */
#define window_bitmap(handle)                  \
        windata[handle].subbitmap

#define window_x(handle)                       \
        windata[handle].x

#define window_y(handle)                       \
        windata[handle].y

#define window_parent(handle)                  \
        windata[handle].parent

#define window_value(handle)                   \
        windata[handle].value

#define copy_var(source,dest)                  \
        memcpy (dest, source, sizeof(source))

#define copy_var_size(source,dest,size)        \
        memcpy (dest, source, size)

#define copy_var_malloc(source,dest)           \
        memcpy (dest = malloc(sizeof(source)), source, sizeof(source))

#define copy_var_malloc_size(source,dest,size) \
        memcpy (dest = malloc(size), source, size)

/* Do the other includes */
#include "guiunhid.h" /* The Window Hiding / Unhiding Code */
#include "gui_tbmo.h" /* Temporary Bitmap Overlays */
#include "guitheme.h" /* GUI theme manager code */
#include "guidialg.h" /* Dialog Routines */

#endif

