/*
 * Temporary Bitmap Overlays for the GUI - code file
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU LGPL version 2.
 *
 * See the file "COPYING.LIB" for software license details.
 */

#include "def.h" /* Also defines the Hide / Unhides */

int tbmo_handle[MAX_GUI_TBMOS]; /* The TBMO allocate order array */
int tbmo_num = 0; /* Number of overlays */
bool overlayson = TRUE; /* Are the overlays on-screen */
GUI_TBMO_STRUCT tbmo_data[MAX_GUI_TBMOS]; /* Overlay Data Array */

/* Bring a TBMO number to the front */
bool bring_tbmo_to_front(int ovrly_hnd) {
  int i, last;
  bool flag = TRUE;

  if (!tbmo_num) return TRUE;
  i = last = (tbmo_num - 1);
  if (ovrly_hnd == tbmo_handle[i]) return FALSE; /* Already at front */
   do {
    i--;
    if (i < 0) return TRUE; /* Not found */
    if (ovrly_hnd == tbmo_handle[i]) flag = FALSE; /* Handle found */
   } while (flag);
   /* Now bring the handle to the front */
   for (; i < last; i++) {
    tbmo_handle[i] = tbmo_handle[i + 1];
   }
  tbmo_handle[last] = ovrly_hnd;
  return FALSE;
 }

/*
 * Place an overlay onto the screen
 *
 *  x       = the x offset from the corner of overlay (from ox)
 *  ya      = the y offset from the corner of overlay -- if above
 *  yb      = the y offset from the corner of overlay -- if below
 *  h       = the h width of the overlay
 *  v       = the v width of the overlay
 *  above   = is above is perferred
 *  copy    = do copy the bitmap in ownbitmap and make local copy.
 *  blitown = blit the window now.
 *
 */
int place_overlay (BITMAP *bitmap, int handle, int x, int ya, int yb,
                   int h, int v, bool above, bool copy, bool blitnow) {
  int py, tx, tya, ty, ovrly_hnd;

  if (tbmo_num >= MAX_GUI_TBMOS) return -1; /* Allocated Enough */
  ovrly_hnd = tbmo_handle [tbmo_num]; /* Get handle number */

 /* Get the point in the window */
  get_screen_point (handle, x, 0, &tx, &py);
  if (tx < 0) tx = 0; /* Make sure that the overlay is on-screen */
  if (tx > scrn_widt - h) tx = scrn_widt - h;
 /* Get the best vertical position */
  tya = (py + ya - v) + 1;
  ty = ((py + yb + v > scrn_hght) ? tya : py + yb);
  if ((tbmo_data[ovrly_hnd].above = above) && tya >= 0) ty = tya;
 /* Get the screen behind it */
   if (!(tbmo_data[ovrly_hnd].background = create_bitmap(h,v))) {
    return -1; } /* Could not allocate bitmap */
  scare_mouse (); /* Hide the mouse cursor */
  blit(screen, tbmo_data[ovrly_hnd].background, tx, ty, 0, 0, h, v);
  unscare_mouse (); /* Unhide the mouse cursor */
 /* Create its own bitmap */
   if (bitmap == NULL || copy) {
     if (!(tbmo_data[ovrly_hnd].overlay = create_bitmap(h,v))) {
      destroy_bitmap(tbmo_data[ovrly_hnd].background);
      return -1;
     }
    if (copy) blit(bitmap, tbmo_data[ovrly_hnd].overlay, 0, 0, 0, 0, h, v);
    tbmo_data[ovrly_hnd].ownbitmap = FALSE;
   } else {
    tbmo_data[ovrly_hnd].ownbitmap = TRUE;
    tbmo_data[ovrly_hnd].overlay = bitmap;
   }
  tbmo_data[ovrly_hnd].h = h;    tbmo_data[ovrly_hnd].v = v;
  tbmo_data[ovrly_hnd].x = tx;
  tbmo_data[ovrly_hnd].y = ty;
  tbmo_data[ovrly_hnd].px = x;   tbmo_data[ovrly_hnd].pya = ya;
  tbmo_data[ovrly_hnd].pyb = yb; tbmo_data[ovrly_hnd].handle = handle;
  tbmo_num++;

  /* blit the bitmap now if blitnow is set */
   if (blitnow) {
    scare_mouse ();
    blit (tbmo_data[ovrly_hnd].overlay, screen, 0, 0, tx, ty, h, v);
    unscare_mouse ();
   }

  return ovrly_hnd; /* Return the handle of overlay */
 }

/* Blit the temporary bitmap overlay on screen */
void blit_tbmo (int ovrly_hnd) {
  if (overlayson) { /* Only do this when the overlays are on */
   if (ovrly_hnd == tbmo_handle[tbmo_num - 1]) { /* Is it at the TOP? */
    scare_mouse ();
    blit (tbmo_data[ovrly_hnd].overlay, screen, 0, 0,
     tbmo_data[ovrly_hnd].x, tbmo_data[ovrly_hnd].y,
     tbmo_data[ovrly_hnd].h, tbmo_data[ovrly_hnd].v);
    unscare_mouse ();
   } else { /* Work on an optimized method later */
    scare_mouse ();
    temporary_hide_tbmos ();
    redraw_all_tbmos ();
    unscare_mouse ();
   }
  }
 }

/* Get the pointer for a bitmap overlay */
BITMAP *overlay_bitmap (int ovrly_hnd) {
  return tbmo_data[ovrly_hnd].overlay;
 }

/* Get the mouse position that is on the overlay */
bool get_mouse_on_overlay (int ovrly_hnd, int *x, int *y) {
   if (overlayson) {
    if ((*x = mouse_x - tbmo_data[ovrly_hnd].x) >= tbmo_data[ovrly_hnd].h) { return TRUE; }
    if ((*y = mouse_y - tbmo_data[ovrly_hnd].y) >= tbmo_data[ovrly_hnd].v) { return TRUE; }
    if (*x < 0 || *y < 0) { return TRUE; }
   } else {
    return TRUE;
   }
  return FALSE;
 }

/* Temporary hide all TBMOs */
void temporary_hide_tbmos () {
  int i, h;

  if (!tbmo_num || !overlayson) return;
  overlayson = FALSE;
  /* Unblit all TBMOs in the reverse order */
   for (i = tbmo_num - 1; i >= 0; i--) {
     h = tbmo_handle[i];
     blit(tbmo_data[h].background, screen, 0, 0, tbmo_data[h].x,
          tbmo_data[h].y, tbmo_data[h].h, tbmo_data[h].v);
    }
 }

/* Redraw all TBMOs that were hidden */
void redraw_all_tbmos () {
  int py, tx, tya, ty, hh, vv, i, h, ox, oy;

  if (!tbmo_num || overlayson) return;
  overlayson = TRUE;
  /* Unblit all TBMOs in the reverse order */
   for (i = 0; i < tbmo_num; i++) {
    h = tbmo_handle[i];
   /* Get the positions */
    hh = tbmo_data[h].h; vv = tbmo_data[h].v;
   /* Calculate the new position */
    get_screen_point (tbmo_data[h].handle, tbmo_data[h].px, 0, &tx, &py);
    if (tx < 0) tx = 0; /* Make sure that the overlay is on-screen */
    if (tx > scrn_widt - hh) tx = scrn_widt - hh;
   /* Get the best vertical position */
    tya = (py + tbmo_data[h].pya - vv) + 1;
    ty = ((py + tbmo_data[h].pyb + vv) > scrn_hght ? tya : py + tbmo_data[h].pyb);
    if (tbmo_data[h].above && tya >= 0) ty = tya;
    tbmo_data[h].x = tx; tbmo_data[h].y = ty;
   /* Copy the background */
     blit(screen, tbmo_data[h].background, tx, ty, 0, 0, hh, vv);
   /* Blit the sprite */
     blit(tbmo_data[h].overlay, screen, 0, 0, tx, ty, hh, vv);
    }
 }

/* Destroy an overlay (whilst the others are on screen) */
bool destroy_overlay (int ovrly_hnd) {
 int h, i;

 /* If there is any TBMOs being allocated */
 if (!tbmo_num) return TRUE;
 /* Check if TBMOs are on screen */
  if (!overlayson) {
    scare_mouse ();
    if (bring_tbmo_to_front (ovrly_hnd)) { unscare_mouse (); return TRUE; }
    destroy_bitmap(tbmo_data[h].background);
    if (!tbmo_data[h].ownbitmap) destroy_bitmap(tbmo_data[h].overlay);
    unscare_mouse (); return FALSE;
   }
 /* Do it the "dirty" method */
  scare_mouse ();
  i = tbmo_num - 1;
  h = tbmo_handle[i];
   if (h == ovrly_hnd) { /* Is the overlay at the top (?) */
    blit(tbmo_data[h].background, screen, 0, 0, tbmo_data[h].x,
         tbmo_data[h].y, tbmo_data[h].h, tbmo_data[h].v);
    destroy_bitmap(tbmo_data[h].background);
    if (!tbmo_data[h].ownbitmap) destroy_bitmap(tbmo_data[h].overlay);
    tbmo_num--;
    unscare_mouse (); return FALSE;
   }
  /* Go down through the layers */
   for (;;) {
    blit(tbmo_data[h].background, screen, 0, 0, tbmo_data[h].x,
         tbmo_data[h].y, tbmo_data[h].h, tbmo_data[h].v);
    if (h == ovrly_hnd) break;
    i--; h = tbmo_handle[i];
   }
 /* Bring the discarded window forward */
  bring_tbmo_to_front(ovrly_hnd); tbmo_num--;
 /* Destroy the discarded window */
  blit(tbmo_data[h].background, screen, 0, 0, tbmo_data[h].x,
       tbmo_data[h].y, tbmo_data[h].h, tbmo_data[h].v);
  destroy_bitmap(tbmo_data[h].background);
  if (!tbmo_data[h].ownbitmap) destroy_bitmap(tbmo_data[h].overlay);
 /* Reblit all the TBMOs */
   for (; i < tbmo_num; i++) {
    h = tbmo_handle[i];
   /* Copy the background */
     blit(screen, tbmo_data[h].background, tbmo_data[h].x, tbmo_data[h].y,
          0, 0, tbmo_data[h].h, tbmo_data[h].v);
   /* Blit the sprite */
     blit(tbmo_data[h].overlay, screen, 0, 0, tbmo_data[h].x,
          tbmo_data[h].y, tbmo_data[h].h, tbmo_data[h].v);
   }
  unscare_mouse (); return FALSE;
 }

