/*
 * Code File for GUI Theme Customization Code
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU LGPL version 2.
 *
 * See the file "COPYING.LIB" for software license details.
 */

#include "def.h"

/* The Wallpaper */
 char *gui_wallpaper = "Magetutu.PCX";

/* Convert between 8-bit palettes */
 void conv_eight_bit_pal (BITMAP *bitmap, RGB *pal) {
  int i, sze, cnt = 0;
  UBYTE *dat, convtable[256];

  /* Create Conversion Table */
   for (i = 0; i < 256; i++) {
    convtable[i] = bestfit_color (set_gui_palette, pal[i].r, pal[i].g, pal[i].b);
   }
  /* Convert the image */
   sze = bitmap->w * bitmap->h; dat = bitmap->dat;
    while (cnt < sze) {
     dat[cnt] = convtable[dat[cnt]]; cnt++;
    }
 }

/* This routine sets up the wallpaper */
 void wallpaper_setup () {
  BITMAP *bitmap; PALETTE pal; UBYTE *bmdat;
  int h, v, hh, vv, px, py;

  /* Can a wallpaper be loaded */
   if (!gui_wallpaper) {
    if (wallpaper) destroy_bitmap (wallpaper);
    wallpaper = NULL;
    return;
   }
  /* Create the wallpaper */
   if (!wallpaper) {
    if (!(wallpaper = create_bitmap (scrn_widt, scrn_hght))) return;
   }
  /* Colour the entire screen with the wall paper colour */
  clear_to_color (wallpaper, clrsh_wallpaper);
  /* Place the wallpaper */
  set_color_conversion (COLORCONV_TOTAL);
#ifdef DEBUG
   fprintf(stderr, "guitheme.c: wallpaper_setup is about to read file \"%s\".\n", gui_wallpaper);
#endif
    if (bitmap = load_bitmap (gui_wallpaper, pal)) {
     h = bitmap->w; v = bitmap->h;
     /* Allegro does not cross convert palettes in 256 images */
     if (bitmap->vtable->color_depth == 8) conv_eight_bit_pal (bitmap, pal);
     /* Just love cross multiplication */
      if ((scrn_widt * v / h) <= scrn_widt) {
       hh = scrn_widt; vv = scrn_widt * v / h;
       px = 0; py = (scrn_hght - vv) >> 1;
      } else {
       vv = scrn_hght; hh = scrn_hght * h / v;
       py = 0; px = (scrn_widt - hh) >> 1;
      }
#ifdef DEBUG
     fprintf(stderr, "guitheme.c: wallpaper_setup has read a bitmap with size %u by %u.\n", h, v);
#endif
     stretch_blit(bitmap, wallpaper, 0, 0, h, v, px, py, hh, vv);
     destroy_bitmap(bitmap);
    } else {
     destroy_bitmap(wallpaper);
#ifdef DEBUG
     fprintf(stderr, "guitheme.c: wallpaper_setup has failed to read the wallpaper.\n");
#endif
    }
 }

