/*
 * MagicTutu's Window Hider / Unhider Routines
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU LGPL version 2.
 *
 * See the file "COPYING.LIB" for software license details.
 *
 */

#include "gui.h"

/* This procedure rolls up a window */
bool rollup_window (int handle) {
  if (windata[handle].flags & WF_INMINSTATE) return FALSE; /* The Window is already minimized */
  if (deactive_window(handle)) return TRUE; /* Check if window can be deactivated */

  return TRUE;
 }

/* This procedure minimizes a window */
bool minimize_window (int handle) {

  if (windata[handle].flags & WF_INMINSTATE) return FALSE; /* The Window is already minimized */
  if (deactive_window(handle)) return TRUE; /* Check if window can be deactivated */

  windata[handle].h_org = windata[handle].h; windata[handle].x_org = windata[handle].x;
  windata[handle].v_org = windata[handle].v; windata[handle].y_org = windata[handle].y;
  windata[handle].y = -1; windata[handle].flags |= WF_INMINSTATE;

  destroy_bitmap (windata[handle].subbitmap); /* Deallocate Subbitmap */
  destroy_bitmap (windata[handle].bitmap); /* Deallocate Bitmap resources */
  windata[handle].bitmap = windata[handle].subbitmap = NULL; /* Nullify pointers */
  refresh_screen_area (windata[handle].x_org, windata[handle].y_org,
                       windata[handle].h_org, windata[handle].v_org);
  return FALSE;
 }

/* This procedure restores a minimized window */
bool restore_window (int handle) {

#ifdef DEBUG_GUI
 fprintf(stderr, "guiunhid.c: restore_window is being executed at: %U\n", gui_timer);
#endif

  if (!(windata[handle].flags & WF_INMINSTATE)) return FALSE; /* The window is not in the minimized state */
  return windata[handle].redraw (handle, windata[handle].value);
 }

/* This procedure restores a minimized window, but on a redraw */
bool restore_window_on_redraw (int handle) {

#ifdef DEBUG_GUI
 fprintf(stderr, "guiunhid.c: restore_window_on_redraw is being executed at: %U\n", gui_timer);
#endif

  if (!(windata[handle].flags & WF_INMINSTATE)) return FALSE;
   if (windata[handle].bitmap = create_bitmap (windata[handle].h_org, windata[handle].v_org)) {
    windata[handle].flags &= ~WF_INMINSTATE;
    windata[handle].h = windata[handle].h_org; windata[handle].x = windata[handle].x_org;
    windata[handle].v = windata[handle].v_org; windata[handle].y = windata[handle].y_org;
#ifdef DEBUG_GUI
    fprintf(stderr, "guiunhid.c: exitting restore_window_on_redraw with FALSE at: %U\n", gui_timer);
#endif
    return FALSE;
   }

#ifdef DEBUG_GUI
 fprintf(stderr, "guiunhid.c: exitting restore_window_on_redraw with TRUE at: %U\n", gui_timer);
#endif
  return TRUE;
 }

/* List all minimized windows */

